#include "stm32f0xx_conf.h"
#include "stm32f0xx.h"
#include "delay.h"
#include "it_conf.h"
#include "rcc_conf.h"
#include "gpio_conf.h"
#include "tim_conf.h"
#include "spi_conf.h"
#include "adc_conf.h"

#define IL_USRED_ADC 100 //ilo usrednien
#define SCALE_LOW 0.24414 //warto skalujca potencjometr LOW, aby wynosila od 0 do 1000
#define SCALE_HIGH 0.24414 //warto skalujca potencjometr HIGH, aby wynosila od 0 do 1000

volatile uint8_t cnt_disp = 0; //0, 1 lub 2 - kolejne wyswietlacze
volatile uint8_t digits[3] = {7, 3, 1}; //cyfry do wystawienia na wyswietlacz
volatile uint8_t color_disp = 0; //0 zielony, 1 zolty, 2 czerwony
volatile uint8_t cnt_col = 0; //0 lub 1 - przelacza kolory przy wyswietlaniu zoltego
volatile uint32_t disp_copy; //do wyswietlenia - kopia
volatile uint8_t poz_led = 0; //poziom diod wieccych

volatile uint32_t pobr_adc = 0;
volatile uint32_t usred_adc = 0;
volatile uint16_t cnt_usred = 0;
volatile uint32_t wynik_adc;
volatile uint8_t wynik_gotowy; //ustawia flage gotowosci wyniku, aby 1 raz dokonac przeliczenia

volatile uint16_t pot_low = 0;
volatile uint16_t pot_high = 0; //wartoci potencjometrow

volatile uint8_t tryb_adc = 0; //0 - praca normalna, 1 - pot low, 2 - pot high

const uint8_t tabl_jasn[8] = {99, 70, 50, 30, 15, 10, 5, 1};
volatile uint8_t cnt_jasn = 0; //wskanik na pozycj jasnoci w tablicy

///////////////////////////////////////////////////
// Wyniki z ADC
///////////////////////////////////////////////////

void ADC1_COMP_IRQHandler(void)
{
	uint32_t adc;
	ADC_ClearITPendingBit(ADC1, ADC_IT_EOC);
	adc = ADC_GetConversionValue(ADC1);
	if(adc > pobr_adc){pobr_adc = adc;} //aktualizuj jezeli nowa wartosc jest wieksza, w ten sposob wykrywa gorke
}

///////////////////////////////////////////////////
// Odswiezanie wyswietlaczy
///////////////////////////////////////////////////

void TIM3_IRQHandler(void)
{
	TIM_ClearITPendingBit(TIM3, TIM_IT_Update);
		usred_adc += pobr_adc; //dodaj poprzednia wartosc aby uzyskac usrednienie
		cnt_usred++;
		pobr_adc = 0;
	if(cnt_usred >= IL_USRED_ADC)
	{
		cnt_usred = 0;
		if(tryb_adc == 0){ wynik_adc = usred_adc / IL_USRED_ADC; }
		if(tryb_adc == 1){ pot_low = usred_adc / IL_USRED_ADC; }
		if(tryb_adc == 2){ pot_high = usred_adc / IL_USRED_ADC; }
		usred_adc = 0;
		wynik_gotowy = 1;
	}

}
///////////////////////////////////////////////////
// Odswiezanie wyswietlaczy
///////////////////////////////////////////////////

void TIM2_IRQHandler(void)
{
	TIM_ClearITPendingBit(TIM2, TIM_IT_Update);
	cnt_disp++;
	if(cnt_disp >= 3){cnt_disp = 0;}
	GPIO_SetBits(GPIOA, GPIO_Pin_15);//A1R //wylacz wyswietlacze
	GPIO_SetBits(GPIOB, GPIO_Pin_3);//A2R
	GPIO_SetBits(GPIOB, GPIO_Pin_4);//A3R
	GPIO_SetBits(GPIOB, GPIO_Pin_5);//A1G
	GPIO_SetBits(GPIOB, GPIO_Pin_6);//A2G
	GPIO_SetBits(GPIOB, GPIO_Pin_7);//A3G

	send_display(digits[cnt_disp], poz_led); //ustaw nowe wartosci segmentow

	switch(color_disp)
	{
	case 0:
		switch(cnt_disp)
		{
		case 0: GPIO_ResetBits(GPIOB, GPIO_Pin_5); break;
		case 1: GPIO_ResetBits(GPIOB, GPIO_Pin_6); break;
		case 2: GPIO_ResetBits(GPIOB, GPIO_Pin_7); break;
		}
	break;

	case 1:
		cnt_col ^= 1; //zmienia wartosc na przeciwna
		if(cnt_col)
		{
			switch(cnt_disp)
			{
				case 0: GPIO_ResetBits(GPIOA, GPIO_Pin_15); break;
				case 1: GPIO_ResetBits(GPIOB, GPIO_Pin_3); break;
				case 2: GPIO_ResetBits(GPIOB, GPIO_Pin_4); break;
			}
		}
		if(!cnt_col)
		{
			switch(cnt_disp)
			{
				case 0: GPIO_ResetBits(GPIOB, GPIO_Pin_5); break;
				case 1: GPIO_ResetBits(GPIOB, GPIO_Pin_6); break;
				case 2: GPIO_ResetBits(GPIOB, GPIO_Pin_7); break;
			}
		}
	break;

	case 2:
		switch(cnt_disp)
		{
		case 0: GPIO_ResetBits(GPIOA, GPIO_Pin_15); break;
		case 1: GPIO_ResetBits(GPIOB, GPIO_Pin_3); break;
		case 2: GPIO_ResetBits(GPIOB, GPIO_Pin_4); break;
		}
	break;


	}

}

///////////////////////////////////////////////////
// Przycisk BRIGHTNESS
///////////////////////////////////////////////////

void EXTI4_15_IRQHandler(void)
{
	if(EXTI_GetITStatus(EXTI_Line14) != RESET) //przycisk
	{
		if(CheckDelay(0) == 0)
		{
			cnt_jasn++;
			if(cnt_jasn >= 8){cnt_jasn = 0;}
			TIM_SetCompare2(TIM15, tabl_jasn[cnt_jasn]);
			SetDelayTime(0, 100); //przerwa przed nastepnym wykonaniem - dla unikniecia drgan stykow
		}
		EXTI_ClearITPendingBit(EXTI_Line14);

	}

}


int main(void)
{

	//inicjalizacje
		SystemInit();
		SystemCoreClockUpdate();
		rcc_conf(); //inicjalizacja zegarow
		it_conf(); //inicjaliacja przerwan
		tim_conf(); //inicjalizacja timerow
		gpio_conf(); //inicjalizacja GPIO
		spi_conf();
		adc_conf();

		TIM_SetCompare2(TIM15, tabl_jasn[cnt_jasn]); //aktualizacja jasnoci tym, co jest w tablicy

		digits[0] = 0; //dla unikniecia glupot na wyswietlaczu
		digits[0] = 0;
		digits[0] = 0;

		change_adc(1); //pobierz wartosc potencjometru LOW
		tryb_adc = 1;
		while(!pot_low){} //zapisuj wartosci

		change_adc(2); //pobierz wartosc potencjometru HIGH
		tryb_adc = 2;
		while(!pot_high){} //zapisuj wartosci

		change_adc(0);
		tryb_adc = 0;

    	  while(1)
    	    {
    			if(GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_12) == Bit_SET && GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_13) == Bit_SET)
    			{
    				change_adc(0);
    				tryb_adc = 0;

    				if(wynik_gotowy)
    				{
    				if(wynik_adc > (pot_low * SCALE_LOW)){ wynik_adc -= (pot_low * SCALE_LOW); } else {wynik_adc = 0;} //odjecie od wyniku skladowej stalej
    				wynik_adc *= 100; //przeskalowanie aby sie lepiej dzielilo
    				wynik_adc /= (pot_high * SCALE_HIGH); //podzielenie

    				if(wynik_adc < 50){color_disp = 0;} //zielony
    				if(wynik_adc >= 50 && wynik_adc < 200){color_disp = 1;} //zolty
    				if(wynik_adc >= 200){color_disp = 2;} //czerwony

    				if(wynik_adc == 0){poz_led = 0;}
    				if(wynik_adc > 0 && wynik_adc <= 10){poz_led = 1;}
    				if(wynik_adc > 10 && wynik_adc <= 25){poz_led = 2;}
    				if(wynik_adc > 25 && wynik_adc <= 50){poz_led = 3;}
    				if(wynik_adc > 50 && wynik_adc <= 100){poz_led = 4;}
    				if(wynik_adc > 100 && wynik_adc <= 200){poz_led = 5;}
    				if(wynik_adc > 200 && wynik_adc <= 300){poz_led = 6;}
    				if(wynik_adc > 300 && wynik_adc <= 400){poz_led = 7;}
    				if(wynik_adc > 400 && wynik_adc <= 500){poz_led = 8;}

    				disp_copy = wynik_adc;
    				if(disp_copy > 999){disp_copy = 999;} //ograniczenie do nasycenia aby nie pokazywac glupot
    				digits[0] = disp_copy / 100;
    				disp_copy -= (digits[0] * 100);
    				digits[1] = disp_copy / 10;
    				disp_copy -= (digits[1] * 10);
    				digits[2] = disp_copy;
    				wynik_gotowy = 0;
    				}
    			}

    			if(GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_12) == Bit_RESET && GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_13) == Bit_SET)
    			{ //pot LOW
    				change_adc(1);
    				tryb_adc = 1;
    				color_disp = 0; //zielony
    				poz_led = 0; //ledy wylaczone

    				if(wynik_gotowy)
    				{
    				disp_copy = pot_low * SCALE_LOW;

    				if(disp_copy > 999){disp_copy = 999;} //ograniczenie do nasycenia aby nie pokazywac glupot
    				digits[0] = disp_copy / 100;
    				disp_copy -= (digits[0] * 100);
    				digits[1] = disp_copy / 10;
    				disp_copy -= (digits[1] * 10);
    				digits[2] = disp_copy;
    				wynik_gotowy = 0;
    				}
    			}

    			if(GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_12) == Bit_SET && GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_13) == Bit_RESET)
    			{ //pot HIGH
    				change_adc(2);
    				tryb_adc = 2;
    				color_disp = 2; //czerwony
    				poz_led = 0; //ledy wylaczone

    				if(pot_high < (100/SCALE_HIGH)){pot_high = ((100/SCALE_HIGH)+1);} //ograniczenie, aby nie bylo dzielenia przez 0
    				disp_copy = pot_high * SCALE_HIGH;

    				if(wynik_gotowy)
    				{
    				if(disp_copy > 999){disp_copy = 999;} //ograniczenie do nasycenia aby nie pokazywac glupot
    				digits[0] = disp_copy / 100;
    				disp_copy -= (digits[0] * 100);
    				digits[1] = disp_copy / 10;
    				disp_copy -= (digits[1] * 10);
    				digits[2] = disp_copy;
    				wynik_gotowy = 0;
    				}
    			}

    	    }
}
